<?php
namespace AkTest\Lib\Type;

use PHPUnit\Framework\TestCase;

/**
 * Class EnumTest
 *
 * @package AkTest\Lib\Type
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/lib
 */
final class EnumTest extends TestCase
{
    public function testWillUseTheDefaultValue()
    {
        $enum = new MockEnum();
        $this->assertEquals(MockEnum::DEFAULT, (string) $enum);
    }

    public function testWillThrowExceptionsForInvalidValueType()
    {
        $exceptionCount = 0;

        try {
            new MockEnum(true);
        } catch (\InvalidArgumentException $e) {
            $exceptionCount++;
        }

        try {
            new MockEnum([]);
        } catch (\InvalidArgumentException $e) {
            $exceptionCount++;
        }

        try {
            new MockEnum(new \stdClass());
        } catch (\InvalidArgumentException $e) {
            $exceptionCount++;
        }

        $this->assertEquals(3, $exceptionCount);
    }

    /**
     * @expectedException \InvalidArgumentException
     */
    public function testWillThrowExceptionForInvalidValue()
    {
        new MockEnum('invalid');
    }

    public function testCanReturnAllValues()
    {
        $this->assertEquals(['red', 'green', 'blue'], MockEnum::getValues());
    }

    public function testCanUseMagicMethods()
    {
        $red = MockEnum::RED();

        $this->assertInstanceOf(MockEnum::class, $red);
        $this->assertEquals(MockEnum::RED, $red);
    }

    /**
     * @expectedException \BadMethodCallException
     */
    public function testWillThrowExceptionForInvalidMagicMethod()
    {
        /** @noinspection PhpUndefinedMethodInspection */
        MockEnum::INVALID();
    }

    public function testCanUseMultipleEnums()
    {
        new MockEnum();
        new MockDirection();
        $this->addToAssertionCount(1);
    }
}